/*****************************************************************************
   callbacks.c - an Alsa based gtk mixer
   Written by Raistlinn (lansdoct@cs.alfred.edu)
   Copyright (C) 1998 by Christopher Lansdown
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

/*****************************************************************************/
/* Begin #include's */

#include "main.h"

/* End #include's */
/*****************************************************************************/

/*****************************************************************************/
/* Begin Global Variables */

extern GtkWidget *window;
extern Card *card; /* And array of the cards */
extern int cards; /* The number of cards in the system. */

/* End Global Variables */
/*****************************************************************************/



void volume_adjust (GtkWidget *widget, CardInfo *info)
{
  GtkWidget *adj;
  GtkWidget *button;
  int err;

  adj = button = NULL;
  
  if(info->flags & CHANNEL_LEFT || info->flags & CHANNEL_RIGHT)
    adj = widget;
  if(info->flags & CHANNEL_MUTE_RIGHT ||
     info->flags & CHANNEL_MUTE_LEFT ||
     info->flags & CHANNEL_RECORD ||
     info->flags & CHANNEL_SIMULTANEOUS)
    button = widget;
  
  if(info->flags & CHANNEL_LEFT || info->flags & CHANNEL_RIGHT)
    if(info->chan->flags & CHANNEL_SIMULTANEOUS) {
      info->chan->data.left = 100 - (int)GTK_ADJUSTMENT (adj)->value;
      info->chan->data.right = 100 - (int)GTK_ADJUSTMENT (adj)->value;
      GTK_ADJUSTMENT (info->other)->value = GTK_ADJUSTMENT (adj)->value;
      gtk_signal_emit_by_name (GTK_OBJECT (info->other), "changed");
    }
    else {
      if(info->flags & CHANNEL_LEFT)
	info->chan->data.left = 100 - (int)GTK_ADJUSTMENT (adj)->value;
      
      if(info->flags & CHANNEL_RIGHT)
	info->chan->data.right = 100 - (int)GTK_ADJUSTMENT (adj)->value;
    }

  if(info->flags & CHANNEL_MUTE_RIGHT || info->flags & CHANNEL_MUTE_LEFT)
    if(info->chan->flags & CHANNEL_SIMULTANEOUS)
      if(GTK_TOGGLE_BUTTON (button)->active) {
	info->chan->data.flags |= SND_MIXER_FLG_MUTE_RIGHT;
	info->chan->data.flags |= SND_MIXER_FLG_MUTE_LEFT;
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (info->other), TRUE);
      }
      else {
	info->chan->data.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
	info->chan->data.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (info->other), FALSE);
      }
    else {
      if(info->flags & CHANNEL_MUTE_RIGHT) 
	if(GTK_TOGGLE_BUTTON (button)->active)
	  info->chan->data.flags |= SND_MIXER_FLG_MUTE_RIGHT;
	else
	  info->chan->data.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
      
      if(info->flags & CHANNEL_MUTE_LEFT)
	if(GTK_TOGGLE_BUTTON (button)->active)
	  info->chan->data.flags |= SND_MIXER_FLG_MUTE_LEFT;
	else
	  info->chan->data.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    }
  
  
  if(info->flags & CHANNEL_RECORD)
    if(GTK_TOGGLE_BUTTON (button)->active)
      info->chan->data.flags |= SND_MIXER_FLG_RECORD;
    else
      info->chan->data.flags &= ~SND_MIXER_FLG_RECORD;
  

  if(info->flags & CHANNEL_SIMULTANEOUS)
    if(GTK_TOGGLE_BUTTON (button)->active)
      info->chan->flags |= CHANNEL_SIMULTANEOUS;
    else
      info->chan->flags &= ~CHANNEL_SIMULTANEOUS;

  /* Write whatever we've done to the kernel */
  err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &info->chan->data);
  if(err < 0)
    printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
}

#ifdef dog
#ifndef dog
void volume_adjust_old (GtkWidget *widget, CardInfo *info)
{
  GtkWidget *adj;
  GtkWidget *button;
  int err;

  adj = button = NULL;
  
  if(info->flags & CHANNEL_LEFT || info->flags & CHANNEL_RIGHT)
    adj = widget;
  if(info->flags & CHANNEL_MUTE_RIGHT ||
     info->flags & CHANNEL_MUTE_LEFT ||
     info->flags & CHANNEL_RECORD)
    button = widget;
  
  if(info->card->master_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->master.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->master.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT) 
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->master.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->master.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->master.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->master.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      info->card->master.flags |= SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->master));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
  }
  if(info->card->bass_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->bass.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->bass.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT) 
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->bass.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->bass.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->bass.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->bass.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->bass.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->bass.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->bass));
    if(err < 0)
      printf("Unable to write bass information to card.\n");
    }
  if(info->card->treble_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->treble.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->treble.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->treble.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->treble.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->treble.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->treble.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->treble.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->treble.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->treble));
    if(err < 0)
      printf("Unable to write treble information to card.\n");
    };
  if(info->card->synthesizer_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->synthesizer.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->synthesizer.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->synthesizer.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->synthesizer.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->synthesizer.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->synthesizer.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->synthesizer.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->synthesizer.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->synthesizer));
    if(err < 0)
      printf("Unable to write synthesizer information to card.\n");
    }
  if(info->card->synthesizer1_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->synthesizer1.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->synthesizer1.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->synthesizer1.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->synthesizer1.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->synthesizer1.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->synthesizer1.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->synthesizer1.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->synthesizer1.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->synthesizer1));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }
  if(info->card->fm_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->fm.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->fm.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->fm.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->fm.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->fm.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->fm.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->fm.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->fm.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->fm));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }
  if(info->card->effect_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->effect.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->effect.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->effect.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->effect.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->effect.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->effect.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->effect.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->effect.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->effect));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }
  if(info->card->pcm_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->pcm.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->pcm.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->pcm.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->pcm.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->pcm.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->pcm.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->pcm.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->pcm.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->pcm));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }
  if(info->card->pcm1_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->pcm1.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->pcm1.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->pcm1.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->pcm1.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->pcm1.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->pcm1.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->pcm1.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->pcm1.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->pcm1));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }
  if(info->card->line_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->line.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->line.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->line.flags |= SND_MIXER_FLG_MUTE_RIGHT;
    else
      info->card->line.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->line.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->line.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->line.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->line.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->line));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }
  if(info->card->mic_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->mic.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->mic.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->mic.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->mic.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->mic.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->mic.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->mic.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->mic.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->mic));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }
  if(info->card->cd_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->cd.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->cd.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->cd.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->cd.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->cd.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->cd.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->cd.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->cd.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->cd));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }
  if(info->card->gain_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->gain.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->gain.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->gain.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->gain.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->gain.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->gain.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->gain.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->gain.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->gain));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }
  if(info->card->igain_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->igain.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->igain.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->igain.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->igain.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->igain.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->igain.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->igain.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->igain.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->igain));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }
  if(info->card->ogain_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->ogain.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->ogain.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->ogain.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->ogain.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->ogain.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->ogain.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->ogain.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->ogain.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->ogain));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }
  if(info->card->loopback_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->loopback.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->loopback.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->loopback.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->loopback.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->loopback.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->loopback.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->loopback.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->loopback.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->loopback));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }
  if(info->card->speaker_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->speaker.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->speaker.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->speaker.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->speaker.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->speaker.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->speaker.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->speaker.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->speaker.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->speaker));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }
  if(info->card->auxa_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->auxa.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->auxa.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->auxa.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->auxa.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->auxa.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->auxa.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->auxa.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->auxa.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->auxa));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }
  if(info->card->auxb_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->auxb.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->auxb.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->auxb.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->auxb.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->auxb.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->auxb.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->auxb.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->auxb.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->auxb));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }
  if(info->card->auxc_num == info->channel) {
    if(info->flags & CHANNEL_LEFT)
      info->card->auxc.left = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_RIGHT)
      info->card->auxc.right = (int)GTK_ADJUSTMENT (adj)->value;
    if(info->flags & CHANNEL_MUTE_RIGHT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->auxc.flags |= SND_MIXER_FLG_MUTE_RIGHT;
      else
	info->card->auxc.flags &= ~SND_MIXER_FLG_MUTE_RIGHT;
    if(info->flags & CHANNEL_MUTE_LEFT)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->auxc.flags |= SND_MIXER_FLG_MUTE_LEFT;
      else
	info->card->auxc.flags &= ~SND_MIXER_FLG_MUTE_LEFT;
    if(info->flags & CHANNEL_RECORD)
      if(GTK_TOGGLE_BUTTON (button)->active)
	info->card->auxc.flags |= SND_MIXER_FLG_RECORD;
      else
	info->card->auxc.flags &= ~SND_MIXER_FLG_RECORD;
    err = snd_mixer_channel_write(info->card->handle, 
				info->channel, &(info->card->auxc));
    if(err < 0)
      printf("Unable to write information to card.  The error was: %s\n", snd_strerror(err));
    }

  return;
}

#endif
#endif
