/*****************************************************************************
   cards.c - an Alsa based gtk mixer
   Written by Raistlinn (lansdoct@cs.alfred.edu)
   Copyright (C) 1998 by Christopher Lansdown
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

/*****************************************************************************/
/* Begin #include's */

#include "main.h"

/* End #include's */
/*****************************************************************************/

/*****************************************************************************/
/* Begin Global Variables */

extern Card *card; /* And array of the cards */
extern int cards; /* The number of cards in the system. */

/* End Global Variables */
/*****************************************************************************/


int init_cards()
{
  /* Begin variable declarations */
  int i;
  int err;
  /* Begin variable declarations */

  /* Get the number of sound cards in the system */
  cards = snd_cards();

  /* Allocate out the handles */
  card = calloc(cards, sizeof(Card));

  /* Open up the mixers and get the mixer info */
  for(i = 0; i < cards; i++) {
    err = snd_mixer_open(&(card[i].handle), i, 0);
    if(err < 0) {
      card[i].handle = NULL;
      printf("Unable to open soundcard %i.  Error: %s\n", i, 
	     snd_strerror(err) );
    }
    if(card[i].handle)
      snd_mixer_info(card[i].handle, &card[i].info);
  }


  /* Get the numbers of the channels */
  for(i = 0; i < cards; i++) {
    card[i].master.num = snd_mixer_channel(card[i].handle, 
					   SND_MIXER_ID_MASTER);
    card[i].bass.num = snd_mixer_channel(card[i].handle, 
					 SND_MIXER_ID_BASS);
    card[i].treble.num = snd_mixer_channel(card[i].handle, 
					   SND_MIXER_ID_TREBLE);
    card[i].synthesizer.num = snd_mixer_channel(card[i].handle, 
						SND_MIXER_ID_SYNTHESIZER);
    card[i].synthesizer1.num = snd_mixer_channel(card[i].handle, 
						 SND_MIXER_ID_SYNTHESIZER1);
    card[i].fm.num = snd_mixer_channel(card[i].handle, SND_MIXER_ID_FM);
    card[i].effect.num = snd_mixer_channel(card[i].handle, 
					   SND_MIXER_ID_EFFECT);
    card[i].pcm.num = snd_mixer_channel(card[i].handle, SND_MIXER_ID_PCM);
    card[i].pcm1.num = snd_mixer_channel(card[i].handle, SND_MIXER_ID_PCM1);
    card[i].line.num = snd_mixer_channel(card[i].handle, SND_MIXER_ID_LINE);
    card[i].mic.num = snd_mixer_channel(card[i].handle, SND_MIXER_ID_MIC);
    card[i].cd.num = snd_mixer_channel(card[i].handle, SND_MIXER_ID_CD);
    card[i].gain.num = snd_mixer_channel(card[i].handle, SND_MIXER_ID_GAIN);
    card[i].igain.num = snd_mixer_channel(card[i].handle, SND_MIXER_ID_IGAIN);
    card[i].ogain.num = snd_mixer_channel(card[i].handle, SND_MIXER_ID_OGAIN);
    card[i].loopback.num = snd_mixer_channel(card[i].handle, 
					     SND_MIXER_ID_LOOPBACK);
    card[i].speaker.num = snd_mixer_channel(card[i].handle, 
					    SND_MIXER_ID_SPEAKER);
    card[i].auxa.num = snd_mixer_channel(card[i].handle, SND_MIXER_ID_AUXA);
    card[i].auxb.num = snd_mixer_channel(card[i].handle, SND_MIXER_ID_AUXB);
    card[i].auxc.num = snd_mixer_channel(card[i].handle, SND_MIXER_ID_AUXC);
    card[i].mono.num = snd_mixer_channel(card[i].handle, SND_MIXER_ID_MONO);
    card[i].mono1.num = snd_mixer_channel(card[i].handle, SND_MIXER_ID_MONO1);
    card[i].mono2.num = snd_mixer_channel(card[i].handle, SND_MIXER_ID_MONO2);
  }



   /* Get the Channel info */
  for(i = 0; i < cards; i++) {
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].master.num, 
				 &card[i].master.info);
    err = snd_mixer_channel_info(card[i].handle,
				 card[i].bass.num,
				 &card[i].bass.info);
    err = snd_mixer_channel_info(card[i].handle,
				 card[i].treble.num,
				 &card[i].treble.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].synthesizer.num,
				 &card[i].synthesizer.info);
    err = snd_mixer_channel_info(card[i].handle,
				 card[i].synthesizer1.num,
				 &card[i].synthesizer1.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].fm.num, 
				 &card[i].fm.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].effect.num,
				 &card[i].effect.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].pcm.num, 
				 &card[i].pcm.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].pcm1.num,
				 &card[i].pcm1.info);
    err = snd_mixer_channel_info(card[i].handle,
				 card[i].line.num  ,
				 &card[i].line.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].mic.num,
				 &card[i].mic.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].cd.num,
				 &card[i].cd.info);
    err = snd_mixer_channel_info(card[i].handle,
				 card[i].gain.num,
				 &card[i].gain.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].igain.num,
				 &card[i].igain.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].ogain.num,
				 &card[i].ogain.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].loopback.num,
				 &card[i].loopback.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].speaker.num,
				 &card[i].speaker.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].auxa.num,
				 &card[i].auxa.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].auxb.num,
				 &card[i].auxb.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].auxc.num,
				 &card[i].auxc.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].mono.num,
				 &card[i].mono.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].mono1.num,
				 &card[i].mono1.info);
    err = snd_mixer_channel_info(card[i].handle, 
				 card[i].mono2.num,
				 &card[i].mono2.info);
  }

  /* Read the channel data in */
  for(i = 0; i < cards; i++) {
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].master.num, 
				 &card[i].master.data);
    err = snd_mixer_channel_read(card[i].handle,
				 card[i].bass.num,
				 &card[i].bass.data);
    err = snd_mixer_channel_read(card[i].handle,
				 card[i].treble.num,
				 &card[i].treble.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].synthesizer.num,
				 &card[i].synthesizer.data);
    err = snd_mixer_channel_read(card[i].handle,
				 card[i].synthesizer1.num,
				 &card[i].synthesizer1.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].fm.num, 
				 &card[i].fm.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].effect.num,
				 &card[i].effect.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].pcm.num, 
				 &card[i].pcm.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].pcm1.num,
				 &card[i].pcm1.data);
    err = snd_mixer_channel_read(card[i].handle,
				 card[i].line.num  ,
				 &card[i].line.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].mic.num,
				 &card[i].mic.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].cd.num,
				 &card[i].cd.data);
    err = snd_mixer_channel_read(card[i].handle,
				 card[i].gain.num,
				 &card[i].gain.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].igain.num,
				 &card[i].igain.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].ogain.num,
				 &card[i].ogain.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].loopback.num,
				 &card[i].loopback.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].speaker.num,
				 &card[i].speaker.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].auxa.num,
				 &card[i].auxa.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].auxb.num,
				 &card[i].auxb.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].auxc.num,
				 &card[i].auxc.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].mono.num,
				 &card[i].mono.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].mono1.num,
				 &card[i].mono1.data);
    err = snd_mixer_channel_read(card[i].handle, 
				 card[i].mono2.num,
				 &card[i].mono2.data);


  }


  return 1;
}
