/*****************************************************************************
   xamixer.c - an Alsa based gtk mixer
   Written by Raistlinn (lansdoct@cs.alfred.edu)
   Copyright (C) 1998 by Christopher Lansdown
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

/*****************************************************************************/
/* Begin #include's */

#include "main.h"

/* End #include's */
/*****************************************************************************/

/*****************************************************************************/
/* Begin Global Variables */

GtkWidget *window;
Card *card; /* And array of the cards */
int cards; /* The number of cards in the system. */

/* End Global Variables */
/*****************************************************************************/

int main(int argc, char **argv)
{
  /* Begin Variable Declarations */
  GtkWidget *mainbox;
  GtkWidget *notebook;
  GtkWidget *frame;
  GtkWidget *label;
  int i;
  char title[32];
  /* End Variable Declarations */

  /* Go through gtk initialization */
  gtk_init(&argc, &argv);

  /* Read in the soundcard info */
  if(!init_cards()) {
    printf("Error.  Unable to initialize sound cards.\n");
    return 1;
  }

  /* Make the title */
  sprintf(title, "XAmixer %s", VERSION);

  /* Create the main window */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), title);
  gtk_signal_connect(GTK_OBJECT (window), "delete_event", 
		     (GtkSignalFunc) gtk_main_quit, NULL);
  signal(SIGINT, (void *)gtk_main_quit);

  /* Create the container box */
  mainbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(mainbox);
  gtk_container_add(GTK_CONTAINER (window), mainbox);

  /* Create the notebook */
  notebook = gtk_notebook_new();
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
  gtk_box_pack_start(GTK_BOX (mainbox), notebook, TRUE, TRUE, 0);
  gtk_widget_show(notebook);

  
  /* Create the notebook pages */
  for(i = 0; i < cards; i++) {
    frame = gtk_frame_new("");
    gtk_widget_show(frame);

    mainbox = gtk_hbox_new(FALSE, 10);
    gtk_container_add(GTK_CONTAINER (frame), mainbox);
    gtk_widget_show(mainbox);


    /* Create the master volume control */
    if(card[i].master.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("Master", 
					       card[i].master.data.left,
					       card[i].master.data.right,
					       &card[i],
					       card[i].master.num,
					       &card[i].master.data,
					       &card[i].master.info,
					       &card[i].master),
			 TRUE, TRUE, 0);

    if(card[i].bass.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("Bass", 
					       card[i].bass.data.left,
					       card[i].bass.data.right,
					       &card[i],
					       card[i].bass.num,
					       &card[i].bass.data,
					       &card[i].bass.info,
					       &card[i].bass),
			 TRUE, TRUE, 0);

    if(card[i].treble.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("Treble", 
					       card[i].treble.data.left,
					       card[i].treble.data.right,
					       &card[i],
					       card[i].treble.num,
					       &card[i].treble.data,
					       &card[i].treble.info,
					       &card[i].treble),
			 TRUE, TRUE, 0);


    if(card[i].synthesizer.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("Synth", 
					       card[i].synthesizer.data.left,
					       card[i].synthesizer.data.right,
					       &card[i],
					       card[i].synthesizer.num,
					       &card[i].synthesizer.data,
					       &card[i].synthesizer.info,
					       &card[i].synthesizer),
			 TRUE, TRUE, 0);

    if(card[i].synthesizer1.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("Synth1", 
					       card[i].synthesizer1.data.left,
					       card[i].synthesizer1.data.right,
					       &card[i],
					       card[i].synthesizer1.num,
					       &card[i].synthesizer1.data,
					       &card[i].synthesizer1.info,
					       &card[i].synthesizer1),
			 TRUE, TRUE, 0);

    if(card[i].fm.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("FM", 
					       card[i].fm.data.left,
					       card[i].fm.data.right,
					       &card[i],
					       card[i].fm.num,
					       &card[i].fm.data,
					       &card[i].fm.info,
					       &card[i].fm),
			 TRUE, TRUE, 0);

    if(card[i].effect.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("Effect", 
					       card[i].effect.data.left,
					       card[i].effect.data.right,
					       &card[i],
					       card[i].effect.num,
					       &card[i].effect.data,
					       &card[i].effect.info,
					       &card[i].effect),
			 TRUE, TRUE, 0);

    if(card[i].pcm.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("PCM", 
					       card[i].pcm.data.left,
					       card[i].pcm.data.right,
					       &card[i],
					       card[i].pcm.num,
					       &card[i].pcm.data,
					       &card[i].pcm.info,
					       &card[i].pcm),
			 TRUE, TRUE, 0);

    if(card[i].pcm1.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("PCM1", 
					       card[i].pcm1.data.left,
					       card[i].pcm1.data.right,
					       &card[i],
					       card[i].pcm1.num,
					       &card[i].pcm1.data,
					       &card[i].pcm1.info,
					       &card[i].pcm1),
			 TRUE, TRUE, 0);

    if(card[i].line.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("LineIn", 
					       card[i].line.data.left,
					       card[i].line.data.right,
					       &card[i],
					       card[i].line.num,
					       &card[i].line.data,
					       &card[i].line.info,
					       &card[i].line),
			 TRUE, TRUE, 0);

    if(card[i].mic.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("MIC", 
					       card[i].mic.data.left,
					       card[i].mic.data.right,
					       &card[i],
					       card[i].mic.num,
					       &card[i].mic.data,
					       &card[i].mic.info,
					       &card[i].mic),
			 TRUE, TRUE, 0);

    if(card[i].cd.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("CD", 
					       card[i].cd.data.left,
					       card[i].cd.data.right,
					       &card[i],
					       card[i].cd.num,
					       &card[i].cd.data,
					       &card[i].cd.info,
					       &card[i].cd),
			 TRUE, TRUE, 0);

    if(card[i].gain.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("Gain", 
					       card[i].gain.data.left,
					       card[i].gain.data.right,
					       &card[i],
					       card[i].gain.num,
					       &card[i].gain.data,
					       &card[i].gain.info,
					       &card[i].gain),
			 TRUE, TRUE, 0);

    if(card[i].igain.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("IGain", 
					       card[i].igain.data.left,
					       card[i].igain.data.right,
					       &card[i],
					       card[i].igain.num,
					       &card[i].igain.data,
					       &card[i].igain.info,
					       &card[i].igain),
			 TRUE, TRUE, 0);

    if(card[i].ogain.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("OGain", 
					       card[i].ogain.data.left,
					       card[i].ogain.data.right,
					       &card[i],
					       card[i].ogain.num,
					       &card[i].ogain.data,
					       &card[i].ogain.info,
					       &card[i].ogain),
			 TRUE, TRUE, 0);

    if(card[i].loopback.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("Loopback", 
					       card[i].loopback.data.left,
					       card[i].loopback.data.right,
					       &card[i],
					       card[i].loopback.num,
					       &card[i].loopback.data,
					       &card[i].loopback.info,
					       &card[i].loopback),
			 TRUE, TRUE, 0);

    if(card[i].speaker.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("Spkr", 
					       card[i].speaker.data.left,
					       card[i].speaker.data.right,
					       &card[i],
					       card[i].speaker.num,
					       &card[i].speaker.data,
					       &card[i].speaker.info,
					       &card[i].speaker),
			 TRUE, TRUE, 0);

    if(card[i].auxa.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("AUXa", 
					       card[i].auxa.data.left,
					       card[i].auxa.data.right,
					       &card[i],
					       card[i].auxa.num,
					       &card[i].auxa.data,
					       &card[i].auxa.info,
					       &card[i].auxa),
			 TRUE, TRUE, 0);

    if(card[i].auxb.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("AUXb", 
					       card[i].auxb.data.left,
					       card[i].auxb.data.right,
					       &card[i],
					       card[i].auxb.num,
					       &card[i].auxb.data,
					       &card[i].auxb.info,
					       &card[i].auxb),
			 TRUE, TRUE, 0);

    if(card[i].auxc.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("AUXc", 
					       card[i].auxc.data.left,
					       card[i].auxc.data.right,
					       &card[i],
					       card[i].auxc.num,
					       &card[i].auxc.data,
					       &card[i].auxc.info,
					       &card[i].auxc),
			 TRUE, TRUE, 0);

    if(card[i].mono.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("Mono", 
					       card[i].mono.data.left,
					       card[i].mono.data.right,
					       &card[i],
					       card[i].mono.num,
					       &card[i].mono.data,
					       &card[i].mono.info,
					       &card[i].mono),
			 TRUE, TRUE, 0);

    if(card[i].mono1.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("Mono1", 
					       card[i].mono1.data.left,
					       card[i].mono1.data.right,
					       &card[i],
					       card[i].mono1.num,
					       &card[i].mono1.data,
					       &card[i].mono1.info,
					       &card[i].mono1),
			 TRUE, TRUE, 0);

    if(card[i].mono1.num >= 0)
      gtk_box_pack_start(GTK_BOX (mainbox), 
			 create_volume_control("Mono1", 
					       card[i].mono1.data.left,
					       card[i].mono1.data.right,
					       &card[i],
					       card[i].mono1.num,
					       &card[i].mono1.data,
					       &card[i].mono1.info,
					       &card[i].mono1),
			 TRUE, TRUE, 0);

    /* Create the label and add the page to the notebook */
    label = gtk_label_new(card[i].info.name);
    gtk_widget_show(label);
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, label);

  }


  /* Show the window */
  gtk_widget_show(window);

  /* And go to the main event loop */
  gtk_main();




  return 0;
}


GtkWidget *create_volume_control(char *name, float left, float right, Card *card, int channel, snd_mixer_channel_t *data, snd_mixer_channel_info_t *chan_info, Channel *schan)
{
  GtkWidget *box, *box1, *box2;
  GtkWidget *rscale, *lscale;
  GtkObject *ladj, *radj; 
  GtkWidget *label;
  CardInfo *info;
  GtkWidget *lmute, *rmute;
  GtkWidget *button;

  rmute = lmute = NULL;

  box = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(box);
  
  /* Create the control label */
  label = gtk_label_new(name);
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX (box), label, FALSE, FALSE, 0);

  /* Make the box to hold the volume range controls and the mute buttons*/
  box1 = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(box1);
  gtk_box_pack_start(GTK_BOX (box), box1, FALSE, FALSE, 0);
  
  /********************/
  /* The Widgets *****/

  /*****************/
  /* The left side */

  /* Make the box to hold each individual range and mute button */
  box2 = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(box2);
  gtk_box_pack_start(GTK_BOX (box1), box2, TRUE, TRUE, 0);

  /* Start with the left volume control */
  ladj = gtk_adjustment_new(101.0 - left, 0.0, 101.0, 2.0, 10.0, 10.0);
  lscale = gtk_vscale_new(GTK_ADJUSTMENT (ladj));
  gtk_widget_show(lscale);
  gtk_scale_set_draw_value(GTK_SCALE (lscale), FALSE);
  gtk_box_pack_start(GTK_BOX (box2), lscale, FALSE, FALSE, 0);


  /* Only set up the mute if the channel can be muted */
  if(chan_info->caps & SND_MIXER_CINFO_CAP_MUTE ||
     chan_info->caps & SND_MIXER_CINFO_CAP_HWMUTE) {
  /* Set up the right mute button */
  lmute = gtk_toggle_button_new_with_label("M");
  gtk_widget_show(lmute);
  gtk_box_pack_start(GTK_BOX(box2), lmute, FALSE, FALSE, 0);
  /* Set the appropriate initial state */
  if(data->flags & SND_MIXER_FLG_MUTE_LEFT)
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (lmute), TRUE);
  }




  /******************/
  /* The right side */

  /* Make the box to hold each individual range and mute button */
  box2 = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(box2);
  gtk_box_pack_start(GTK_BOX (box1), box2, TRUE, TRUE, 0);

  /* Create the right volume control */
  radj = gtk_adjustment_new(101.0 - right, 0.0, 101.0, 2.0, 10.0, 10.0);
  rscale = gtk_vscale_new(GTK_ADJUSTMENT (radj));
  gtk_widget_show(rscale);
  gtk_scale_set_draw_value(GTK_SCALE (rscale), FALSE);
  gtk_box_pack_start(GTK_BOX (box2), rscale, FALSE, FALSE, 0);

  /* Only set up the mute if the channel can be muted */
  if(chan_info->caps & SND_MIXER_CINFO_CAP_MUTE ||
     chan_info->caps & SND_MIXER_CINFO_CAP_HWMUTE) {
  /* Set up the left mute button */
    rmute = gtk_toggle_button_new_with_label("M");
    gtk_widget_show(rmute);
    gtk_box_pack_start(GTK_BOX(box2), rmute, FALSE, FALSE, 0);
    /* Set the appropriate initial state */
    if(data->flags & SND_MIXER_FLG_MUTE_RIGHT)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (rmute), TRUE);
  }



  /**********************/
  /* The Callbacks *****/

  /* The left */
  /* The Scale callback */
  info = calloc(1, sizeof(CardInfo));
  info->card = card;
  info->channel = channel;
  info->other = (GtkWidget *)radj;
  info->chan = schan;
  info->flags &= 0;
  info->flags |= CHANNEL_LEFT;
  gtk_signal_connect(GTK_OBJECT (ladj), "value_changed",
		     (GtkSignalFunc) volume_adjust, info);

  /* Only set up the mute if the channel can be muted */
  if(chan_info->caps & SND_MIXER_CINFO_CAP_MUTE ||
     chan_info->caps & SND_MIXER_CINFO_CAP_HWMUTE) {
  /* The Mute callback */
  info = calloc(1, sizeof(CardInfo));
  info->card = card;
  info->channel = channel;
  info->other = rmute;
  info->chan = schan;
  info->flags &= 0;
  info->flags |= CHANNEL_MUTE_LEFT;
  gtk_signal_connect(GTK_OBJECT (lmute), "toggled",
		     (GtkSignalFunc) volume_adjust, info);
  }


  /* The Right */

  /* Set up the Right callback */  
  info = calloc(1, sizeof(CardInfo));
  info->card = card;
  info->channel = channel;
  info->other = (GtkWidget *)ladj;
  info->chan = schan;
  info->flags &= 0;
  info->flags |= CHANNEL_RIGHT;
  gtk_signal_connect(GTK_OBJECT (radj), "value_changed",
		     (GtkSignalFunc) volume_adjust, info);

  /* Set up the Mute callback */
  /* Only set up the mute if the channel can be muted */
  if(chan_info->caps & SND_MIXER_CINFO_CAP_MUTE ||
     chan_info->caps & SND_MIXER_CINFO_CAP_HWMUTE) {
    info = calloc(1, sizeof(CardInfo));
    info->card = card;
    info->channel = channel;
    info->other = lmute;
    info->chan = schan;
    info->flags &= 0;
    info->flags |= CHANNEL_MUTE_RIGHT;
    gtk_signal_connect(GTK_OBJECT (rmute), "toggled",
		     (GtkSignalFunc) volume_adjust, info);
  }


  /***************************************/
  /* Create the simultaneous button *****/
  button = gtk_toggle_button_new_with_label("|-|");
  gtk_widget_show(button);
  gtk_box_pack_start(GTK_BOX (box), button, FALSE, FALSE, 0);
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);
  /* Set up the callback */
  info = calloc(1, sizeof(CardInfo));
  info->card = card;
  info->channel = channel;
  info->chan = schan;
  info->flags &= 0;
  info->flags |= CHANNEL_SIMULTANEOUS;
  gtk_signal_connect(GTK_OBJECT (button), "toggled",
		     (GtkSignalFunc) volume_adjust, info);
  /* And let's not forget to set the simultaneous flag */
  schan->flags |= CHANNEL_SIMULTANEOUS;


  /*********************************/
  /* Create the Record button *****/
  if(chan_info->caps & SND_MIXER_CINFO_CAP_RECORD) {
    button = gtk_toggle_button_new_with_label("Rec");
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX (box), button, FALSE, FALSE, 0);

    /* Set the appropriate initial state */
    if(data->flags & SND_MIXER_FLG_RECORD)
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (button), TRUE);

    /* Set up the callback */
    info = calloc(1, sizeof(CardInfo));
    info->card = card;
    info->channel = channel;
    info->chan = schan;
    info->flags &= 0;
    info->flags |= CHANNEL_RECORD;
    gtk_signal_connect(GTK_OBJECT (button), "toggled",
		       (GtkSignalFunc) volume_adjust, info);
  }




  return box;
}
















